package org.wikipedia.dataclient.okhttp
import okhttp3.Dns
import okhttp3.HttpUrl.Companion.toHttpUrl
import okhttp3.dnsoverhttps.DnsOverHttps
import java.net.InetAddress
const val WIKIMEDIA_DOH_URL = "https://wikimedia-dns.org/dns-query"
data class IpInfo(
    val address: String,
    val isBlocked: Boolean,
    val dataCenter: String? = null
)
// {{main|Help:如何访问维基百科#维基媒体服务器列表}}
// https://zh.wikipedia.org/wiki/Help:如何访问维基百科#维基媒体服务器列表
private val textLbIpsV4 = listOf(
    IpInfo("208.80.154.224", true, "eqiad"),
    IpInfo("208.80.153.224", false, "codfw"),
    IpInfo("198.35.26.96", true, "ulsfo"),
    IpInfo("185.15.59.224", false, "esams"),
    IpInfo("103.102.166.224", true, "eqsin"),
    IpInfo("185.15.58.224", false, "drmrs"),
    IpInfo("195.200.68.224", false, "magru")
)

private val textLbIpsV6 = listOf(
    IpInfo("2620:0:861:ed1a::1", false, "eqiad"),
    IpInfo("2620:0:860:ed1a::1", false, "codfw"),
    IpInfo("2620:0:863:ed1a::1", false, "ulsfo"),
    IpInfo("2a02:ec80:300:ed1a::1", false, "esams"),
    IpInfo("2001:df2:e500:ed1a::1", true, "eqsin"), // Sat May 17 21:34:00 CST 2025
    IpInfo("2a02:ec80:600:ed1a::1", false, "drmrs"),
    IpInfo("2a02:ec80:700:ed1a::1", false, "magru")
)

private val uploadLbIpsV4 = listOf(
    IpInfo("208.80.154.240", false, "eqiad"),
    IpInfo("208.80.153.240", false, "codfw"),
    IpInfo("198.35.26.112", true, "ulsfo"),
    IpInfo("185.15.59.240", false, "esams"),
    IpInfo("103.102.166.240", true, "eqsin"),
    IpInfo("185.15.58.240", false, "drmrs"),
    IpInfo("195.200.68.240", false, "magru")
)

private val uploadLbIpsV6 = listOf(
    IpInfo("2620:0:861:ed1a::2:b", false, "eqiad"),
    IpInfo("2620:0:860:ed1a::2:b", false, "codfw"),
    IpInfo("2620:0:863:ed1a::2:b", false, "ulsfo"),
    IpInfo("2a02:ec80:300:ed1a::2:b", false, "esams"),
    IpInfo("2001:df2:e500:ed1a::2:b", false, "eqsin"),
    IpInfo("2a02:ec80:600:ed1a::2:b", false, "drmrs"),
    IpInfo("2a02:ec80:700:ed1a::2:b", false, "magru")
)

private val dohClient: DnsOverHttps by lazy {
    DnsOverHttps.Builder()
        .client(OkHttpConnectionFactory.client)
        .url(WIKIMEDIA_DOH_URL.toHttpUrl())
        .post(true)
        .bootstrapDnsHosts(
            InetAddress.getByName("185.71.138.138"),
            InetAddress.getByName("2001:67c:930::1")
        )
        .build()
}
object WikimediaDns : Dns {
    override fun lookup(hostname: String): List<InetAddress> {
        return runCatching {
            dohClient.lookup(hostname)
                .map {
                    InetAddress.getByName(when (it.hostAddress) {
                        "103.102.166.224" -> textLbIpsV4.filter { !it.isBlocked }.randomOrNull()?.address
                        "2001:df2:e500:ed1a::1" -> textLbIpsV6.filter { !it.isBlocked }.randomOrNull()?.address
                        "103.102.166.240" -> uploadLbIpsV4.filter { !it.isBlocked }.randomOrNull()?.address
                        else -> it.hostAddress
                    })
                }
                .takeIf { it.isNotEmpty() }
        }.onFailure { it.printStackTrace() }
            .getOrNull() ?: Dns.SYSTEM.lookup(hostname)
    }
}
